/// <reference types="node" />
import * as stream from 'stream';
import TypedEmitter from 'typed-emitter';
export interface TurboDownloaderOptions {
    url: string;
    destFile: string;
    chunkSize: number;
    concurrency: number;
    retryCount: number;
    canBeResumed: boolean;
    adapter: any;
    fillFileByte: number;
    transformStream?: (stream: stream.Readable) => stream.Readable;
}
interface TurboDownloaderConstructorOptions {
    url: string;
    destFile: string;
    chunkSize?: number;
    concurrency?: number;
    retryCount?: number;
    canBeResumed?: boolean;
    adapter?: any;
    fillFileByte?: number;
    transformStream?: (stream: stream.Readable) => stream.Readable;
}
interface DownloadingChunk {
    disposition: number;
    size: number;
    downloaded: number;
}
interface DownloadUrlOptions {
    size: number;
    acceptRanges: boolean;
}
declare type DownloadingPlan = DownloadUrlOptions & {
    chunks: DownloadingChunk[];
};
declare type Events = {
    downloadStarted: (url: string, destination: string) => void;
    downloadFinished: (url: string, destination: string) => void;
    downloadError: (url: string, destination: string) => void;
    chunkDownloadStarted: (chunk: DownloadingChunk, attemptNumber: number) => void;
    chunkDownloadProgress: (chunk: DownloadingChunk) => void;
    chunkDownloadFinished: (chunk: DownloadingChunk, attemptNumber: number) => void;
    chunkDownloadError: (chunk: DownloadingChunk, attemptNumber: number, error: any) => void;
    planReady: (plan: DownloadingPlan) => void;
    aborted: () => void;
    reservingSpaceStarted: (size: number) => void;
    reservingSpaceFinished: (size: number) => void;
};
declare const TurboDownloader_base: new () => TypedEmitter<Events>;
export default class TurboDownloader extends TurboDownloader_base {
    protected options: TurboDownloaderOptions;
    protected started: boolean;
    protected aborted: boolean;
    protected abortSaveProgress: boolean;
    protected abortHandlers: (() => void)[];
    private httpAgent;
    private httpsAgent;
    constructor(options: TurboDownloaderConstructorOptions);
    download(progressCallback?: (downloaded: number, total: number, plan: DownloadingPlan) => void): Promise<void>;
    abort(saveProgress?: boolean): Promise<void>;
    private deletePlanFromDisk;
    private downloadChunk;
    private getDownloadingPlan;
    private getPlanFileName;
    private readDownloadingPlanFromDisk;
    private saveDownloadingPlanToDisk;
    private createDownloadingPlan;
    private getOptions;
    private reserveSpace;
}
export {};
